@@ -37,11 +37,12 @@
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-client.h>
 #include <libgnomevfs/gnome-vfs-init.h>
+#include <eel/eel-gnome-extensions.h>
 #include <stdlib.h>
 #include <string.h>
 
-#define META_FACTORY_IID "OAFIID:nautilus_adapter_factory_generic_factory:8e62e106-807d-4d37-b14a-00dc82ecf88f"
-#define FACTORY_OBJECT_IID    "OAFIID:nautilus_adapter_factory:fd24ecfc-0a6e-47ab-bc53-69d7487c6ad4"
+#define META_FACTORY_IID   "OAFIID:Nautilus_Adapter_Factory_Factory"
+#define FACTORY_OBJECT_IID "OAFIID:Nautilus_Adapter_Factory"
 
 static int object_count = 0;
 
@@ -105,7 +106,7 @@ main (int argc, char *argv[])
 
 	/* Create the factory. */
 
-	registration_id = bonobo_activation_make_registration_id (META_FACTORY_IID, gdk_get_display ());
+	registration_id = eel_bonobo_make_registration_id (META_FACTORY_IID);
 
 	factory = bonobo_generic_factory_new (registration_id, adapter_factory_make_object, NULL);
 
