@@ -43,6 +43,7 @@
 #define EMBLEM_NAME_SYMLINK "emblem-symbolic-link"
 #define EMBLEM_NAME_NOREAD  "emblem-noread"
 #define EMBLEM_NAME_NOWRITE "emblem-nowrite"
+#define EMBLEM_NAME_NOTE    "emblem-note"
 
 GList *
 nautilus_emblem_list_availible (void)
@@ -94,6 +95,9 @@ is_reserved_keyword (const char *keyword)
 	if (eel_strcasecmp (keyword, NAUTILUS_FILE_EMBLEM_NAME_SYMBOLIC_LINK) == 0) {
 		return TRUE;
 	}
+	if (eel_strcasecmp (keyword, NAUTILUS_FILE_EMBLEM_NAME_NOTE) == 0) {
+		return TRUE;
+	}
 
 	availible = nautilus_emblem_list_availible ();
 	icon_name = nautilus_emblem_get_icon_name_from_keyword (keyword);
@@ -121,6 +125,9 @@ nautilus_emblem_should_show_in_list (const char *emblem)
 	if (strcmp (emblem, EMBLEM_NAME_NOWRITE) == 0) {
 		return FALSE;
 	}
+	if (strcmp (emblem, EMBLEM_NAME_NOTE) == 0) {
+		return FALSE;
+	}
 
 	return TRUE;
 }
