@@ -137,6 +137,8 @@ typedef struct {
         guint sweep_timer;
 
 	CacheIcon *fallback_icon;
+	GHashTable *image_mime_types;
+
 } NautilusIconFactory;
 
 #define NAUTILUS_ICON_FACTORY(obj) \
@@ -319,6 +321,14 @@ static void
 nautilus_icon_factory_instance_init (NautilusIconFactory *factory)
 {
 	GdkPixbuf *pixbuf;
+	guint i;
+	static const char *types [] = {
+		"image/x-bmp", "image/x-ico", "image/jpeg", "image/gif",
+		"image/png", "image/pnm", "image/ras", "image/tga",
+		"image/tiff", "image/wbmp", "image/x-xbitmap",
+		"image/x-xpixmap"
+        };
+
 	
         factory->icon_cache = g_hash_table_new_full (cache_key_hash,
 						     cache_key_equal,
@@ -351,6 +361,13 @@ nautilus_icon_factory_instance_init (NautilusIconFactory *factory)
 					   NULL);
 	
 	factory->fallback_icon = cache_icon_new (pixbuf, NULL);
+
+	factory->image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i = 0; i < G_N_ELEMENTS (types); i++) {
+		g_hash_table_insert (factory->image_mime_types,
+				     (gpointer) types [i],
+				     GUINT_TO_POINTER (1));
+	}
 }
 
 static void
@@ -626,6 +643,11 @@ nautilus_icon_factory_finalize (GObject *object)
 		g_assert (factory->fallback_icon->ref_count == 1);
 		cache_icon_unref (factory->fallback_icon);
 	}
+
+	if (factory->image_mime_types) {
+		g_hash_table_destroy (factory->image_mime_types);
+		factory->image_mime_types = NULL;
+	}
 	
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -650,6 +672,10 @@ show_thumbnails_changed_callback (gpointer user_data)
 	show_image_thumbs = eel_preferences_get_enum (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS);
 
 	nautilus_icon_factory_clear ();
+	/* If the user disabled thumbnailing, remove all outstanding thumbnails */ 
+	if (show_image_thumbs == NAUTILUS_SPEED_TRADEOFF_NEVER) {
+		nautilus_thumbnail_remove_all_from_queue ();
+	}
 	g_signal_emit (global_icon_factory,
 		       signals[ICONS_CHANGED], 0);
 }
@@ -702,10 +728,24 @@ image_uri_to_name_or_uri (const char *image_uri)
 }
 
 static gboolean
-should_show_thumbnail (NautilusFile *file)
+mimetype_limited_by_size (const char *mime_type)
 {
-	if (nautilus_file_get_size (file) >
-	    (unsigned int)cached_thumbnail_limit) {
+	NautilusIconFactory *factory;
+
+	factory = get_icon_factory();
+
+        if (g_hash_table_lookup (factory->image_mime_types, mime_type)) {
+                return TRUE;
+	}
+
+        return FALSE;
+}
+
+static gboolean
+should_show_thumbnail (NautilusFile *file, const char *mime_type)
+{
+	if (mimetype_limited_by_size (mime_type) &&
+	    nautilus_file_get_size (file) > (unsigned int)cached_thumbnail_limit) {
 		return FALSE;
 	}
 	
@@ -761,7 +801,7 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file, gboolean embedd_tex
 	
 	file_info = nautilus_file_peek_vfs_file_info (file);
 	
-	show_thumb = should_show_thumbnail (file);	
+	show_thumb = should_show_thumbnail (file, mime_type);	
 	
 	if (show_thumb) {
 		thumb_factory = factory->thumbnail_factory;
