@@ -58,7 +58,7 @@ typedef struct {
 
 } NautilusNavigationBarClass;
 
-GtkType nautilus_navigation_bar_get_type         (void);
+GType   nautilus_navigation_bar_get_type         (void);
 void	nautilus_navigation_bar_activate	 (NautilusNavigationBar *bar);
 char *  nautilus_navigation_bar_get_location     (NautilusNavigationBar *bar);
 void    nautilus_navigation_bar_set_location     (NautilusNavigationBar *bar,
